import logging

LOGGER = logging.getLogger(__name__)


def success_headers(headers):
    """
    Updates headers for a message when a process succeeds

    Resets the kafka_retry_count to 0,
    or adds if it didn't originally exist
    """

    try:
        headers = dict(headers)
    except TypeError:
        headers = {}

    headers['kafka_retry_count'] = '0'
    return headers


def produce_message_callback(error, message):
    """
    Logs the returned message from the Broker after producing
    """
    LOGGER.debug('Callback for producing a message.')
    if error:
        LOGGER.critical(error)


def consume_message_callback(error, partitions):
    """
    Logs the info returned when a successful commit is performed
    """
    LOGGER.debug('Callback for a commit operation from the consumer')
    if error:
        LOGGER.critical(error)
    else:
        LOGGER.debug('No commit errors')



import uuid
from time import sleep
from .general_runtime_vars import env_vars


def generate_guid():
    return str(uuid.uuid1())


def parse_headers(msg_header):
    """
    Converts headers to a dict
    :param msg_header: A message .headers() (confluent) or .headers (faust) instance
    :return: a decoded dict version of the headers
    """
    if msg_header:
        msg_header = dict(msg_header)
        return {key: value.decode() for key, value in msg_header.items()}
    return {}


def log_and_raise_error(metrics_manager, error):
    """
    Since the metric manager pushing is a separate thread, ensure an exception gets sent to prometheus
    """
    metrics_manager.inc_message_errors(error)
    sleep(int(env_vars()['METRICS_PUSH_RATE']) * 2 + 1)
    raise



import sys
from logging import getLogger
from collections import deque
from nubium_utils import init_logger
from nubium_utils.faust_utils.faust_runtime_vars import default_env_vars


def kiwf_log_analysis():
    lines_to_tail = 10
    last_lines = deque(open(default_env_vars()['KIWF_LOG_FILEPATH']), lines_to_tail)
    errors = [error_string for error_string in default_env_vars()['KIWF_ERROR_STRINGS'].split(',') if
              error_string in str(last_lines)]

    if errors:
        init_logger(__name__)
        LOGGER = getLogger(__name__)
        LOGGER.error(f"KIWF encountered error(s): [{', '.join(errors)}].\nKilling Pod.")
        sys.exit(1)



from .consumer_utils import consume_message, commit_message
from .message_utils import success_headers, produce_message_callback, consume_message_callback
from .producer_utils import produce_message, produce_retry_message, produce_failure_message
from .confluent_configs import (init_ssl_configs,
                                init_schema_registry_configs,
                                init_producer_configs,
                                init_consumer_configs,
                                init_metrics_pushing)
